﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Xml;
using System.Configuration;
using System.Web.UI.WebControls;
using EPiServer;
using EPiServer.Core;
using EPiServer.PlugIn;
using EPiServer.Configuration;
using EPiServer.Web.WebControls;
using EPiServer.Filters;

namespace Sitemap
{
    [ScheduledPlugIn(LanguagePath = "/createsitemap")]
    public class Sitemap
    {
        #region "----------------------- Variable declaration -----------------------"

        private static String BaseURL = String.Empty;

        private struct ModulueSettings
        {
            public String Path;
            public String TimeFormat;
        }

        #endregion

        #region "----------------------- Properties -----------------------"

        [PlugInProperty(AdminControl = typeof(TextBox), AdminControlValue = "Text", LanguagePath = "/sitemapsettings/path")]
        public String Path
        {
            get;
            set;
        }

        [PlugInProperty(AdminControl = typeof(TextBox), AdminControlValue = "Text", LanguagePath = "/sitemapsettings/timeformat")]
        public String TimeFormat
        {
            get;
            set;
        }

        #endregion

        #region "----------------------- Functions -----------------------"

        private static String GetBaseUrl()
        {
            String url = Settings.Instance.SiteUrl.ToString();

            if (url.EndsWith("/"))
            {
                return url.Remove(url.LastIndexOf("/"));
            }
            else
            {
                return url;
            }
        }

        private static String GetFriendlyURL(PageData pData, String sLanguage)
        {
            UrlBuilder objUrilBuilder = new UrlBuilder(EPiServer.UriSupport.AddLanguageSelection(pData.LinkURL, sLanguage));

            EPiServer.Global.UrlRewriteProvider.ConvertToExternal(objUrilBuilder, pData.PageLink, System.Text.UTF8Encoding.UTF8);

            if (objUrilBuilder.ToString().Contains("http://"))
            {
                return objUrilBuilder.ToString();
            }

            return BaseURL + objUrilBuilder.ToString();
        }

        private static PageDataCollection GetPages()
        {
            PropertyCriteriaCollection criterias = new PropertyCriteriaCollection();

            PropertyCriteria category = new PropertyCriteria();
            category.Condition = CompareCondition.NotEqual;
            category.Value = "-1";
            category.Type = PropertyDataType.PageType;
            category.Required = true;
            category.Name = "PageTypeID";

            criterias.Add(category);

            PageDataCollection objCollection = DataFactory.Instance.FindPagesWithCriteria(PageReference.StartPage, criterias);

            FilterForVisitor.Filter(objCollection);

            return objCollection;
        }

        private static Int32 CreateSitemap(PageReference Start, String Path, String TimeFormat)
        {
            BaseURL = GetBaseUrl();

            String sTimeFormat = TimeFormat != String.Empty ? TimeFormat : "yyyy-MM-dd";

            PageDataCollection objCollection = new PageDataCollection();

            // Add start page
            objCollection.Add(DataFactory.Instance.GetPage(Start));

            foreach (PageData objPageData in GetPages())
            {
                if (objPageData["PageShortcutType"] == null || (objPageData["PageShortcutType"] != null && ((EPiServer.Core.PageShortcutType)objPageData["PageShortcutType"]) != PageShortcutType.External))
                {
                    objCollection.Add(objPageData);
                }
            }

            XmlDocument xDoc = new XmlDocument();

            XmlNode xNode = xDoc.CreateXmlDeclaration("1.0", "utf-8", null);

            xDoc.AppendChild(xNode);

            XmlNode xURLSet = xDoc.CreateElement("urlset");
            XmlAttribute xXMLNS = xDoc.CreateAttribute("xmlns");
            xXMLNS.Value = "http://www.sitemaps.org/schemas/sitemap/0.9";

            xURLSet.Attributes.Append(xXMLNS);
            xDoc.AppendChild(xURLSet);

            Int32 iCounter = 0;

            foreach (PageData objPageData in objCollection)
            {
                foreach (String sLanguage in objPageData.PageLanguages)
                {
                    XmlNode xURLNode = xDoc.CreateElement("url");
                    XmlNode xLoc = xDoc.CreateElement("loc");
                    XmlNode xURL = xDoc.CreateTextNode(GetFriendlyURL(objPageData, sLanguage));

                    xLoc.AppendChild(xURL);
                    xURLNode.AppendChild(xLoc);

                    XmlNode xLastMod = xDoc.CreateElement("lastmod");
                    xLastMod.AppendChild(xDoc.CreateTextNode(objPageData.Saved.ToString(sTimeFormat)));
                    xURLNode.AppendChild(xLastMod);

                    // Optional change frequency
                    if (objPageData["ChangeFrequency"] != null && objPageData["ChangeFrequency"].ToString() != String.Empty)
                    {
                        XmlNode xChangeFrequency = xDoc.CreateElement("changefreq");
                        xChangeFrequency.AppendChild(xDoc.CreateTextNode(objPageData["ChangeFrequency"].ToString()));
                        xURLNode.AppendChild(xChangeFrequency);
                    }

                    // Optional priority
                    if (objPageData["Priority"] != null && objPageData["Priority"].ToString() != String.Empty)
                    {
                        XmlNode xPriority = xDoc.CreateElement("priority");
                        xPriority.AppendChild(xDoc.CreateTextNode(objPageData["Priority"].ToString()));
                        xURLNode.AppendChild(xPriority);
                    }

                    xURLSet.AppendChild(xURLNode);

                    iCounter++;
                }
            }

            xDoc.Save(Path);

            return iCounter;
        }

        private static ModulueSettings GetSettings()
        {
            Sitemap objSitemap = new Sitemap();
            PlugInSettings.AutoPopulate(objSitemap);

            ModulueSettings objSettings = new ModulueSettings();

            objSettings.Path = objSitemap.Path;
            objSettings.TimeFormat = objSitemap.TimeFormat;

            return objSettings;
        }

        #endregion

        #region "----------------------- Event Procedures -----------------------"

        #endregion

        #region "----------------------- Overrides -----------------------"

        public static String Execute()
        {
            ModulueSettings objSettings = GetSettings();

            if (String.IsNullOrEmpty(objSettings.Path))
            {
                throw new Exception(LanguageManager.Instance.Translate("/createsitemap/errpathnotset"));
            }

            Int32 iURLs = CreateSitemap(PageReference.StartPage, objSettings.Path, objSettings.TimeFormat);

            if (iURLs != -1)
            {
                return String.Format(LanguageManager.Instance.Translate("/createsitemap/message"), iURLs, objSettings.Path);
            }

            return String.Empty;
        }

        #endregion
    }
}